﻿using HarmonyLib;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;

namespace Whitebrim.LeavesAsCompost
{
    public class LeavesAsCompost : Mod
    {
        private const int THATCH_FUEL_VALUE = 5;
        private const int PLANK_FUEL_VALUE = 10;
        private const int FLOWER_FUEL_VALUE = 10;
        private const int FEATHER_FUEL_VALUE = 5;
        private List<SO_FuelValue> fuelValues;
        private List<Item_Base> itemBases;
        public static List<(string, int)> items = new List<(string, int)> { 
            ("Thatch", THATCH_FUEL_VALUE), 
            ("Plank", PLANK_FUEL_VALUE),
            ("Flower_Black", FLOWER_FUEL_VALUE),
            ("Flower_Blue", FLOWER_FUEL_VALUE),
            ("Flower_Red", FLOWER_FUEL_VALUE),
            ("Flower_White", FLOWER_FUEL_VALUE),
            ("Flower_Yellow", FLOWER_FUEL_VALUE),
            ("Feather", FEATHER_FUEL_VALUE),
        };
        Harmony harmonyInstance;

        public void Awake()
        {
            harmonyInstance = new Harmony("com.whitebrim.leavesascompost");
            harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
            CreadeSO();
            if (RAPI.IsCurrentSceneGame())
            {
                AddFuelType();
                PatchAllCompostersManually();
            }
            Debug.Log("LeavesAsCompost has been loaded!");
        }

        private void CreadeSO()
        {
            fuelValues = new List<SO_FuelValue>();
            itemBases = new List<Item_Base>();
            for (int i = 0; i < items.Count; i++)
            {
                var fuel = ScriptableObject.CreateInstance<SO_FuelValue>();
                fuel.itemType = ItemManager.GetItemByName(items[i].Item1);
                itemBases.Add(fuel.itemType);
                fuel.fuelValueOfType = items[i].Item2;
                fuel.fuelFiltrationType = FuelValue_FiltrationType.FV_COMPOSTCOMPOSTTANK;
                fuel.name = $"FuelValue_{items[i].Item1}_CompostTank";
                fuelValues.Add(fuel);
            }
        }

        public override void WorldEvent_WorldLoaded()
        {
            AddFuelType();
        }

        private void AddFuelType()
        {
            var liquidFueelManager = FindObjectOfType<LiquidFuelManager>();
            if (Traverse.Create(liquidFueelManager).Field("initialized").GetValue<bool>())
            {
                LiquidFuelManager.fuelTypesStatic = LiquidFuelManager.fuelTypesStatic.AddRangeToArray(fuelValues.ToArray());
            }
            liquidFueelManager.fuelTypes = liquidFueelManager.fuelTypes.AddRangeToArray(fuelValues.ToArray());
        }

        private void PatchAllCompostersManually()
        {
            var composters = FindObjectsOfType<Placeable_BiofuelExtractor>();
            if (composters != null)
            {
                foreach (var composter in composters)
                {
                    var traverse = typeof(Tank_BiofuelExtractor).BaseType.GetField("acceptableTypes", BindingFlags.Instance | BindingFlags.NonPublic);
                    List<Item_Base> list = traverse.GetValue(composter.compostTank) as List<Item_Base>;
                    list.AddRange(itemBases);
                }
            }
        }

        public void OnModUnload()
        {
            {
                var liquidFueelManager = FindObjectOfType<LiquidFuelManager>();
                var tempList = liquidFueelManager.fuelTypes.ToList();
                for (int i = 0; i < fuelValues.Count; i++)
                {
                    tempList.Remove(fuelValues[i]);
                }
                liquidFueelManager.fuelTypes = tempList.ToArray();
            }
            {
                var tempList = LiquidFuelManager.fuelTypesStatic.ToList();
                for (int i = 0; i < fuelValues.Count; i++)
                {
                    tempList.Remove(fuelValues[i]);
                }
                LiquidFuelManager.fuelTypesStatic = tempList.ToArray();
            }
            var composters = FindObjectsOfType<Placeable_BiofuelExtractor>();
            if (composters != null)
            {
                foreach (var composter in composters)
                {
                    var traverse = typeof(Tank_BiofuelExtractor).BaseType.GetField("acceptableTypes", BindingFlags.Instance | BindingFlags.NonPublic);
                    List<Item_Base> list = traverse.GetValue(composter.compostTank) as List<Item_Base>;
                    for (int i = 0; i < itemBases.Count; i++)
                    {
                        list.Remove(itemBases[i]);
                    }
                }
            }

            harmonyInstance.UnpatchAll();
            Debug.Log("LeavesAsCompost has been unloaded!");
            Destroy(gameObject);
        }
    }

    class Placeable_BiofuelExtractorPatch
    {
        [HarmonyPatch(typeof(Placeable_BiofuelExtractor), "OnBlockPlaced")]
        class OnBlockPlaced
        {
            private static void Prefix(Placeable_BiofuelExtractor __instance, bool ___hasBeenPlaced)
            {
                if (!___hasBeenPlaced)
                {
                    var traverse = typeof(Tank_BiofuelExtractor).BaseType.GetField("acceptableTypes", BindingFlags.Instance | BindingFlags.NonPublic);
                    List<Item_Base> list = traverse.GetValue(__instance.compostTank) as List<Item_Base>;
                    foreach (var item in LeavesAsCompost.items)
                    {
                        list.Add(ItemManager.GetItemByName(item.Item1));
                    }
                }
            }
        }
    }
}